@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
+require_relative "spec_helper"
 
 describe "Blockless Ruby Filters" do
   before do
@@ -13,37 +13,38 @@ describe "Blockless Ruby Filters" do
     end
   end
   
+  it "should support boolean columns directly" do
+    x = Sequel[:x]
+    x.dup.must_be_same_as x
+    x.clone.must_be_same_as x
+  end
+  
   it "should support boolean columns directly" do
     @d.l(:x).must_equal 'x'
   end
   
-  it "should support qualified columns and aliased columns using symbols" do
+  with_symbol_splitting "should support qualified columns and aliased columns using symbols" do
     @d.l(:x__y).must_equal 'x.y'
     @d.l(:x___y).must_equal 'x AS y'
     @d.l(:x__y___z).must_equal 'x.y AS z'
   end
 
-  it "should support qualified columns using virtual rows" do
+  with_symbol_splitting "should support qualified columns using virtual rows" do
     @d.l(Sequel.expr{x__y}).must_equal 'x.y'
   end
 
   it "should not split symbols or virtual row methods if symbol splitting is disabled" do
-    begin
-      Sequel.split_symbols = false
-      @d.l(:x__y).must_equal 'x__y'
-      @d.l(:x___y).must_equal 'x___y'
-      @d.l(:x__y___z).must_equal 'x__y___z'
-      @d.l(Sequel.expr{x__y}).must_equal 'x__y'
-    ensure
-      Sequel.split_symbols = true
-    end
+    @d.l(:x__y).must_equal 'x__y'
+    @d.l(:x___y).must_equal 'x___y'
+    @d.l(:x__y___z).must_equal 'x__y___z'
+    @d.l(Sequel.expr{x__y}).must_equal 'x__y'
   end
 
   it "should support NOT with SQL functions" do
     @d.l(~Sequel.function(:is_blah)).must_equal 'NOT is_blah()'
     @d.l(~Sequel.function(:is_blah, :x)).must_equal 'NOT is_blah(x)'
-    @d.l(~Sequel.function(:is_blah, :x__y)).must_equal 'NOT is_blah(x.y)'
-    @d.l(~Sequel.function(:is_blah, :x, :x__y)).must_equal 'NOT is_blah(x, x.y)'
+    @d.l(~Sequel.function(:is_blah, Sequel[:x][:y])).must_equal 'NOT is_blah(x.y)'
+    @d.l(~Sequel.function(:is_blah, :x, Sequel[:x][:y])).must_equal 'NOT is_blah(x, x.y)'
   end
 
   it "should handle multiple ~" do
@@ -100,28 +101,26 @@ describe "Blockless Ruby Filters" do
     @d.l{(x + y) =~ /blah/}.must_equal '((x + y) ~ \'blah\')'
   end
 
-  if RUBY_VERSION >= '1.9'
-    it "should support != and similar inversions via !~ method" do
-      @d.l{x !~ 100}.must_equal '(x != 100)'
-      @d.l{x !~ 'a'}.must_equal '(x != \'a\')'
-      @d.l{x !~ true}.must_equal '(x IS NOT TRUE)'
-      @d.l{x !~ false}.must_equal '(x IS NOT FALSE)'
-      @d.l{x !~ nil}.must_equal '(x IS NOT NULL)'
-      @d.l{x !~ (1...5)}.must_equal '((x < 1) OR (x >= 5))'
-      @d.l{x !~ [1,2,3]}.must_equal '(x NOT IN (1, 2, 3))'
+  it "should support != and similar inversions via !~ method" do
+    @d.l{x !~ 100}.must_equal '(x != 100)'
+    @d.l{x !~ 'a'}.must_equal '(x != \'a\')'
+    @d.l{x !~ true}.must_equal '(x IS NOT TRUE)'
+    @d.l{x !~ false}.must_equal '(x IS NOT FALSE)'
+    @d.l{x !~ nil}.must_equal '(x IS NOT NULL)'
+    @d.l{x !~ (1...5)}.must_equal '((x < 1) OR (x >= 5))'
+    @d.l{x !~ [1,2,3]}.must_equal '(x NOT IN (1, 2, 3))'
 
-      @d.l{(x + y) !~ 100}.must_equal '((x + y) != 100)'
-      @d.l{(x + y) !~ 'a'}.must_equal '((x + y) != \'a\')'
-      @d.l{(x + y) !~ true}.must_equal '((x + y) IS NOT TRUE)'
-      @d.l{(x + y) !~ false}.must_equal '((x + y) IS NOT FALSE)'
-      @d.l{(x + y) !~ nil}.must_equal '((x + y) IS NOT NULL)'
-      @d.l{(x + y) !~ (1...5)}.must_equal '(((x + y) < 1) OR ((x + y) >= 5))'
-      @d.l{(x + y) !~ [1,2,3]}.must_equal '((x + y) NOT IN (1, 2, 3))'
+    @d.l{(x + y) !~ 100}.must_equal '((x + y) != 100)'
+    @d.l{(x + y) !~ 'a'}.must_equal '((x + y) != \'a\')'
+    @d.l{(x + y) !~ true}.must_equal '((x + y) IS NOT TRUE)'
+    @d.l{(x + y) !~ false}.must_equal '((x + y) IS NOT FALSE)'
+    @d.l{(x + y) !~ nil}.must_equal '((x + y) IS NOT NULL)'
+    @d.l{(x + y) !~ (1...5)}.must_equal '(((x + y) < 1) OR ((x + y) >= 5))'
+    @d.l{(x + y) !~ [1,2,3]}.must_equal '((x + y) NOT IN (1, 2, 3))'
 
-      @d = @d.with_extend{def supports_regexp?; true end}
-      @d.l{x !~ /blah/}.must_equal '(x !~ \'blah\')'
-      @d.l{(x + y) !~ /blah/}.must_equal '((x + y) !~ \'blah\')'
-    end
+    @d = @d.with_extend{def supports_regexp?; true end}
+    @d.l{x !~ /blah/}.must_equal '(x !~ \'blah\')'
+    @d.l{(x + y) !~ /blah/}.must_equal '((x + y) !~ \'blah\')'
   end
   
   it "should support ~ via Hash and Regexp (if supported by database)" do
@@ -187,7 +186,7 @@ describe "Blockless Ruby Filters" do
     @d.lit(1 + Sequel.expr(:x)).must_equal '(1 + x)'
     @d.lit(2**65 - Sequel.+(:x, 1)).must_equal "(#{2**65} - (x + 1))"
     @d.lit(1.0 / Sequel.function(:x)).must_equal '(1.0 / x())'
-    @d.lit(BigDecimal.new('1.0') * Sequel.expr(:a__y)).must_equal '(1.0 * a.y)'
+    @d.lit(BigDecimal.new('1.0') * Sequel[:a][:y]).must_equal '(1.0 * a.y)'
     @d.lit(2 ** Sequel.cast(:x, Integer)).must_equal 'power(2, CAST(x AS integer))'
     @d.lit(1 + Sequel.lit('x')).must_equal '(1 + x)'
     @d.lit(1 + Sequel.lit('?', :x)).must_equal '(1 + x)'
@@ -320,17 +319,6 @@ describe "Blockless Ruby Filters" do
     proc{Sequel::SQL::BooleanExpression.invert(Sequel.expr(:x) + 2)}.must_raise(Sequel::Error)
   end
 
-  it "should return self on .lit" do
-    y = Sequel.expr(:x) + 1
-    y.lit.must_equal y
-  end
-
-  it "should return have .sql_literal return the literal SQL for the expression" do
-    y = Sequel.expr(:x) + 1
-    y.sql_literal(@d).must_equal '(x + 1)'
-    y.sql_literal(@d).must_equal @d.literal(y)
-  end
-
   it "should support SQL::Constants" do
     @d.l({:x => Sequel::NULL}).must_equal '(x IS NULL)'
     @d.l({:x => Sequel::NOTNULL}).must_equal '(x IS NOT NULL)'
@@ -501,13 +489,9 @@ describe "Blockless Ruby Filters" do
   end
 
   it "should handled emulated function where only name is emulated" do
-    db = Sequel.mock
-    db.extend_datasets{}
-    dsc = db.dataset_class
-    efm = dsc::EMULATED_FUNCTION_MAP.dup
-    dsc::EMULATED_FUNCTION_MAP[:trim] = :foo
-    dsc.new(@d.db).literal(Sequel.trim(:a)).must_equal 'foo(a)'
-    dsc::EMULATED_FUNCTION_MAP.replace(efm)
+    ds = Sequel.mock[:a]
+    ds.literal(Sequel.trim(:a)).must_equal 'trim(a)'
+    ds.with_extend{def native_function_name(f) 'foo' end}.literal(Sequel.trim(:a)).must_equal 'foo(a)'
   end
 
   it "should handled emulated function needing full emulation" do
@@ -535,7 +519,7 @@ describe Sequel::SQL::VirtualRow do
     @d.l{column}.must_equal '"column"'
   end
 
-  it "should treat methods without arguments that have embedded double underscores as qualified identifiers" do
+  with_symbol_splitting "should treat methods without arguments that have embedded double underscores as qualified identifiers" do
     @d.l{table__column}.must_equal '"table"."column"'
   end
 
@@ -543,12 +527,13 @@ describe Sequel::SQL::VirtualRow do
     @d.l{function(arg1, 10, 'arg3')}.must_equal 'function("arg1", 10, \'arg3\')'
   end
 
-  it "should treat methods with a block and no arguments as a function call with no arguments" do
-    @d.l{version{}}.must_equal 'version()'
+
+  it "should treat methods followed by function as a function call with no arguments" do
+    @d.l{version.function}.must_equal 'version()'
   end
 
-  it "should treat methods with a block and a leading argument :* as a function call with the SQL wildcard" do
-    @d.l{count(:*){}}.must_equal 'count(*)'
+  it "should treat methods followed by function.* as a function call with * argument" do
+    @d.l{count.function.*}.must_equal 'count(*)'
   end
 
   it "should support * method on functions to raise error if function already has an argument" do
@@ -556,69 +541,60 @@ describe Sequel::SQL::VirtualRow do
   end
 
   it "should support * method on functions to use * as the argument" do
-    @d.l{count{}.*}.must_equal 'count(*)'
+    @d.l{count.function.*}.must_equal 'count(*)'
     @d.literal(Sequel.expr{sum(1) * 2}).must_equal '(sum(1) * 2)'
   end
 
-  it "should treat methods with a block and a leading argument :distinct as a function call with DISTINCT and the additional method arguments" do
-    @d.l{count(:distinct, column1){}}.must_equal 'count(DISTINCT "column1")'
-    @d.l{count(:distinct, column1, column2){}}.must_equal 'count(DISTINCT "column1", "column2")'
-  end
-
   it "should support distinct methods on functions to use DISTINCT before the arguments" do
     @d.l{count(column1).distinct}.must_equal 'count(DISTINCT "column1")'
     @d.l{count(column1, column2).distinct}.must_equal 'count(DISTINCT "column1", "column2")'
   end
 
-  it "should raise an error if an unsupported argument is used with a block" do
-    proc{@d.where{count(:blah){}}}.must_raise(Sequel::Error)
+  it "should handle method.function.over as a window function call" do
+    @d.l{rank.function.over}.must_equal 'rank() OVER ()'
   end
 
-  it "should treat methods with a block and a leading argument :over as a window function call" do
-    @d.l{rank(:over){}}.must_equal 'rank() OVER ()'
+  it "should handle method.function.over(:partition) as a window function call" do
+    @d.l{rank.function.over(:partition=>column1)}.must_equal 'rank() OVER (PARTITION BY "column1")'
+    @d.l{rank.function.over(:partition=>[column1, column2])}.must_equal 'rank() OVER (PARTITION BY "column1", "column2")'
   end
 
-  it "should support :partition options for window function calls" do
-    @d.l{rank(:over, :partition=>column1){}}.must_equal 'rank() OVER (PARTITION BY "column1")'
-    @d.l{rank(:over, :partition=>[column1, column2]){}}.must_equal 'rank() OVER (PARTITION BY "column1", "column2")'
+  it "should handle method(arg).over options as a window function call" do
+    @d.l{avg(column1).over}.must_equal 'avg("column1") OVER ()'
+    @d.l{avg(column1, column2).over}.must_equal 'avg("column1", "column2") OVER ()'
   end
 
-  it "should support :args options for window function calls" do
-    @d.l{avg(:over, :args=>column1){}}.must_equal 'avg("column1") OVER ()'
-    @d.l{avg(:over, :args=>[column1, column2]){}}.must_equal 'avg("column1", "column2") OVER ()'
+  it "should handle method.function.over(:order) as a window function call" do
+    @d.l{rank.function.over(:order=>column1)}.must_equal 'rank() OVER (ORDER BY "column1")'
+    @d.l{rank.function.over(:order=>[column1, column2])}.must_equal 'rank() OVER (ORDER BY "column1", "column2")'
   end
 
-  it "should support :order option for window function calls" do
-    @d.l{rank(:over, :order=>column1){}}.must_equal 'rank() OVER (ORDER BY "column1")'
-    @d.l{rank(:over, :order=>[column1, column2]){}}.must_equal 'rank() OVER (ORDER BY "column1", "column2")'
+  it "should handle method.function.over(:window) as a window function call" do
+    @d.l{rank.function.over(:window=>:win)}.must_equal 'rank() OVER ("win")'
   end
 
-  it "should support :window option for window function calls" do
-    @d.l{rank(:over, :window=>:win){}}.must_equal 'rank() OVER ("win")'
+  it "should handle method.function.*.over as a window function call" do
+    @d.l{count.function.*.over}.must_equal 'count(*) OVER ()'
   end
 
-  it "should support :*=>true option for window function calls" do
-    @d.l{count(:over, :* =>true){}}.must_equal 'count(*) OVER ()'
+  it "should handle method.function.over(:frame=>:all) as a window function call" do
+    @d.l{rank.function.over(:frame=>:all)}.must_equal 'rank() OVER (ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)'
   end
 
-  it "should support :frame=>:all option for window function calls" do
-    @d.l{rank(:over, :frame=>:all){}}.must_equal 'rank() OVER (ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)'
+  it "should handle method.function.over(:frame=>:rows) as a window function call" do
+    @d.l{rank.function.over(:frame=>:rows)}.must_equal 'rank() OVER (ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
   end
 
-  it "should support :frame=>:rows option for window function calls" do
-    @d.l{rank(:over, :frame=>:rows){}}.must_equal 'rank() OVER (ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
-  end
-
-  it "should support :frame=>'some string' option for window function calls" do
-    @d.l{rank(:over, :frame=>'RANGE BETWEEN 3 PRECEDING AND CURRENT ROW'){}}.must_equal 'rank() OVER (RANGE BETWEEN 3 PRECEDING AND CURRENT ROW)'
+  it "should handle method.function.over(:frame=>'some string') as a window function call" do
+    @d.l{rank.function.over(:frame=>'RANGE BETWEEN 3 PRECEDING AND CURRENT ROW')}.must_equal 'rank() OVER (RANGE BETWEEN 3 PRECEDING AND CURRENT ROW)'
   end
 
   it "should raise an error if an invalid :frame option is used" do
-    proc{@d.l{rank(:over, :frame=>:blah){}}}.must_raise(Sequel::Error)
+    proc{@d.l{rank.function.over(:frame=>:blah)}}.must_raise(Sequel::Error)
   end
 
-  it "should support all these options together" do
-    @d.l{count(:over, :* =>true, :partition=>a, :order=>b, :window=>:win, :frame=>:rows){}}.must_equal 'count(*) OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
+  it "should support all over options together" do
+    @d.l{count.function.*.over(:partition=>a, :order=>b, :window=>:win, :frame=>:rows)}.must_equal 'count(*) OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
   end
 
   it "should support order method on functions to specify orders for aggregate functions" do
@@ -626,7 +602,6 @@ describe Sequel::SQL::VirtualRow do
   end
 
   it "should support over method on functions to create window functions" do
-    @d.l{rank{}.over}.must_equal 'rank() OVER ()'
     @d.l{sum(c).over(:partition=>a, :order=>b, :window=>:win, :frame=>:rows)}.must_equal 'sum("c") OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
   end
 
@@ -635,36 +610,36 @@ describe Sequel::SQL::VirtualRow do
   end
 
   it "should raise error if over is called on a function that already has a window " do
-    proc{@d.l{rank{}.over.over}}.must_raise(Sequel::Error)
+    proc{@d.l{rank.function.over.over}}.must_raise(Sequel::Error)
   end
 
   it "should raise an error if window functions are not supported" do
-    proc{@d.with_extend{def supports_window_functions?; false end}.l{count(:over, :* =>true, :partition=>a, :order=>b, :window=>:win, :frame=>:rows){}}}.must_raise(Sequel::Error)
-    proc{Sequel.mock.dataset.filter{count(:over, :* =>true, :partition=>a, :order=>b, :window=>:win, :frame=>:rows){}}.sql}.must_raise(Sequel::Error)
+    proc{@d.with_extend{def supports_window_functions?; false end}.l{count.function.*.over(:partition=>a, :order=>b, :window=>:win, :frame=>:rows)}}.must_raise(Sequel::Error)
+    proc{Sequel.mock.dataset.filter{count.function.*.over(:partition=>a, :order=>b, :window=>:win, :frame=>:rows)}.sql}.must_raise(Sequel::Error)
   end
   
   it "should handle lateral function calls" do
-    @d.l{rank{}.lateral}.must_equal 'LATERAL rank()' 
+    @d.l{rank.function.lateral}.must_equal 'LATERAL rank()' 
   end
 
   it "should handle ordered-set and hypothetical-set function calls" do
-    @d.l{mode{}.within_group(:a)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a")' 
-    @d.l{mode{}.within_group(:a, :b)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a", "b")' 
+    @d.l{mode.function.within_group(:a)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a")' 
+    @d.l{mode.function.within_group(:a, :b)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a", "b")' 
   end
 
   it "should handle filtered aggregate function calls" do
-    @d.l{count{}.*.filter(:a, :b)}.must_equal 'count(*) FILTER (WHERE ("a" AND "b"))' 
-    @d.l{count{}.*.filter(:a=>1)}.must_equal 'count(*) FILTER (WHERE ("a" = 1))'
-    @d.l{count{}.*.filter{b > 1}}.must_equal 'count(*) FILTER (WHERE ("b" > 1))'
-    @d.l{count{}.*.filter(:a=>1){b > 1}}.must_equal 'count(*) FILTER (WHERE (("a" = 1) AND ("b" > 1)))'
+    @d.l{count.function.*.filter(Sequel.&(:a, :b))}.must_equal 'count(*) FILTER (WHERE ("a" AND "b"))' 
+    @d.l{count.function.*.filter(:a=>1)}.must_equal 'count(*) FILTER (WHERE ("a" = 1))'
+    @d.l{count.function.*.filter{b > 1}}.must_equal 'count(*) FILTER (WHERE ("b" > 1))'
+    @d.l{count.function.*.filter(:a=>1){b > 1}}.must_equal 'count(*) FILTER (WHERE (("a" = 1) AND ("b" > 1)))'
   end
 
   it "should handle fitlered ordered-set and hypothetical-set function calls" do
-    @d.l{mode{}.within_group(:a).filter(:a=>1)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a") FILTER (WHERE ("a" = 1))' 
+    @d.l{mode.function.within_group(:a).filter(:a=>1)}.must_equal 'mode() WITHIN GROUP (ORDER BY "a") FILTER (WHERE ("a" = 1))' 
   end
 
   it "should handle function calls with ordinality" do
-    @d.l{foo{}.with_ordinality}.must_equal 'foo() WITH ORDINALITY' 
+    @d.l{foo.function.with_ordinality}.must_equal 'foo() WITH ORDINALITY' 
   end
 
   it "should support function method on identifiers to create functions" do
@@ -673,29 +648,44 @@ describe Sequel::SQL::VirtualRow do
     @d.l{sum.function(c, 1)}.must_equal 'sum("c", 1)'
   end
 
-  it "should support function method on qualified identifiers to create functions" do
+  with_symbol_splitting "should support function method on foo__bar methods to create functions" do
     @d.l{sch__rank.function}.must_equal 'sch.rank()' 
     @d.l{sch__sum.function(c)}.must_equal 'sch.sum("c")'
     @d.l{sch__sum.function(c, 1)}.must_equal 'sch.sum("c", 1)'
+    @d.l{Sequel.qualify(sch[:sum], x[:y]).function(c, 1)}.must_equal 'sch.sum.x.y("c", 1)'
+  end
+
+  it "should support function method on qualified identifiers to create functions" do
+    @d.l{sch[rank].function}.must_equal 'sch.rank()' 
+    @d.l{sch[sum].function(c)}.must_equal 'sch.sum("c")'
+    @d.l{sch[sum].function(c, 1)}.must_equal 'sch.sum("c", 1)'
+    @d.l{Sequel.qualify(sch[:sum], x[:y]).function(c, 1)}.must_equal 'sch.sum.x.y("c", 1)'
+  end
+
+  with_symbol_splitting "should support function method on qualified identifiers to create functions" do
     @d.l{Sequel.qualify(sch__sum, :x__y).function(c, 1)}.must_equal 'sch.sum.x.y("c", 1)'
   end
 
-  it "should handle quoted function names" do
+  it "should not quote function names created from identifiers by default" do
+    @d = @d.with_extend{def supports_quoted_function_names?; true end}
+    @d.l{rank.function}.must_equal 'rank()' 
+  end
+
+  with_symbol_splitting "should handle quoted function names when using double underscores" do
     @d = @d.with_extend{def supports_quoted_function_names?; true end}
-    @d.l{rank.function}.must_equal '"rank"()' 
     @d.l{sch__rank.function}.must_equal '"sch"."rank"()' 
   end
 
   it "should quote function names if a quoted function is used and database supports quoted function names" do
     @d = @d.with_extend{def supports_quoted_function_names?; true end}
-    @d.l{rank{}.quoted}.must_equal '"rank"()' 
-    @d.l{sch__rank{}.quoted}.must_equal '"sch__rank"()' 
+    @d.l{rank(1).quoted}.must_equal '"rank"(1)' 
+    @d.l{rank.function.quoted}.must_equal '"rank"()' 
+    @d.l{sch__rank(1).quoted}.must_equal '"sch__rank"(1)' 
   end
 
-  it "should not quote function names if an unquoted function is used" do
+  it "should not quote function names created from qualified identifiers if an unquoted function is used" do
     @d = @d.with_extend{def supports_quoted_function_names?; true end}
-    @d.l{rank.function.unquoted}.must_equal 'rank()' 
-    @d.l{sch__rank.function.unquoted}.must_equal 'sch.rank()' 
+    @d.l{sch[rank].function.unquoted}.must_equal 'sch.rank()' 
   end
 
   it "should deal with classes without requiring :: prefix" do
@@ -704,23 +694,6 @@ describe Sequel::SQL::VirtualRow do
     @d.l{num < Math::PI.to_i}.must_equal "(\"num\" < 3)"
   end
   
-  it "should deal with methods added to Object after requiring Sequel" do
-    class Object
-      def adsoiwemlsdaf; 42; end
-    end
-    Sequel::BasicObject.remove_methods!
-    @d.l{a > adsoiwemlsdaf}.must_equal '("a" > "adsoiwemlsdaf")'
-  end
-  
-  it "should deal with private methods added to Kernel after requiring Sequel" do
-    module Kernel
-      private
-      def adsoiwemlsdaf2; 42; end
-    end
-    Sequel::BasicObject.remove_methods!
-    @d.l{a > adsoiwemlsdaf2}.must_equal '("a" > "adsoiwemlsdaf2")'
-  end
-
   it "should have operator methods defined that produce Sequel expression objects" do
     @d.l{|o| o.&({:a=>1}, :b)}.must_equal '(("a" = 1) AND "b")'
     @d.l{|o| o.|({:a=>1}, :b)}.must_equal '(("a" = 1) OR "b")'
@@ -736,11 +709,6 @@ describe Sequel::SQL::VirtualRow do
     @d.l{|o| o.<=(1, :b)}.must_equal '(1 <= "b")'
     @d.l{|o| o.>=(1, :b)}.must_equal '(1 >= "b")'
   end
-
-  it "should have have ` produce literal strings" do
-    @d.l{a > `some SQL`}.must_equal '("a" > some SQL)'
-    @d.l{|o| o.a > o.`('some SQL')}.must_equal '("a" > some SQL)' #`
-  end
 end
 
 describe "Sequel core extension replacements" do
@@ -766,6 +734,9 @@ describe "Sequel core extension replacements" do
     Sequel.expr{|o| o.a}.must_be_kind_of(Sequel::SQL::Identifier)
     Sequel.expr{a}.must_be_kind_of(Sequel::SQL::Identifier)
     Sequel.expr(:a).must_be_kind_of(Sequel::SQL::Identifier)
+  end
+
+  with_symbol_splitting "Sequel.expr should return items wrapped in Sequel objects for splittable symbols" do
     Sequel.expr(:a__b).must_be_kind_of(Sequel::SQL::QualifiedIdentifier)
     Sequel.expr(:a___c).must_be_kind_of(Sequel::SQL::AliasedExpression)
     Sequel.expr(:a___c).expression.must_be_kind_of(Sequel::SQL::Identifier)
@@ -796,12 +767,12 @@ describe "Sequel core extension replacements" do
 
   it "Sequel.expr should treat blocks/procs as virtual rows and wrap the output" do
     l(Sequel.expr{1} + 1, "(1 + 1)")
-    l(Sequel.expr{o__a} + 1, "(o.a + 1)")
+    l(Sequel.expr{o[a]} + 1, "(o.a + 1)")
     l(Sequel.expr{[[:a, 1]]} & nil, "((a = 1) AND NULL)")
     l(Sequel.expr{|v| @o} + 1, "(foo + 1)")
 
     l(Sequel.expr(proc{1}) + 1, "(1 + 1)")
-    l(Sequel.expr(proc{o__a}) + 1, "(o.a + 1)")
+    l(Sequel.expr(proc{o[a]}) + 1, "(o.a + 1)")
     l(Sequel.expr(proc{[[:a, 1]]}) & nil, "((a = 1) AND NULL)")
     l(Sequel.expr(proc{|v| @o}) + 1, "(foo + 1)")
   end
@@ -966,13 +937,9 @@ describe "Sequel core extension replacements" do
     o = Sequel.blob(('a'..'z').to_a.join)
     o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=26 start=\"abcdefghij\" end=\"qrstuvwxyz\">"
     o = Sequel.blob(255.chr)
-    o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=1 content=\"#{RUBY_VERSION >= '1.9' ? "\\xFF" : "\\377"}\">"
+    o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=1 content=\"\\xFF\">"
     o = Sequel.blob((230..255).map(&:chr).join)
-    if RUBY_VERSION >= '1.9'
-      o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=26 start=\"\\xE6\\xE7\\xE8\\xE9\\xEA\\xEB\\xEC\\xED\\xEE\\xEF\" end=\"\\xF6\\xF7\\xF8\\xF9\\xFA\\xFB\\xFC\\xFD\\xFE\\xFF\">"
-    else
-      o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=26 start=\"\\346\\347\\350\\351\\352\\353\\354\\355\\356\\357\" end=\"\\366\\367\\370\\371\\372\\373\\374\\375\\376\\377\">"
-    end
+    o.inspect.must_equal "#<Sequel::SQL::Blob:0x#{'%x' % o.object_id} bytes=26 start=\"\\xE6\\xE7\\xE8\\xE9\\xEA\\xEB\\xEC\\xED\\xEE\\xEF\" end=\"\\xF6\\xF7\\xF8\\xF9\\xFA\\xFB\\xFC\\xFD\\xFE\\xFF\">"
   end
 
   it "Sequel.deep_qualify should do a deep qualification into nested structors" do
@@ -1051,6 +1018,10 @@ describe "Sequel core extension replacements" do
     l(Sequel.subscript(:a, 1...3), 'a[1:2]')
   end
 
+  it "Sequel.subscript.f should be subscript expression for backwards compatibility" do
+    Sequel.subscript(:a, 1).f.must_equal :a
+  end
+
   it "Sequel.function should return an SQL function" do
     l(Sequel.function(:a), 'a()')
     l(Sequel.function(:a, 1), 'a(1)')
@@ -1063,7 +1034,7 @@ describe "Sequel core extension replacements" do
 
   it "#* with no arguments should use a ColumnAll for Identifier and QualifiedIdentifier" do
     l(Sequel.expr(:a).*, 'a.*')
-    l(Sequel.expr(:a__b).*, 'a.b.*')
+    l(Sequel[:a][:b].*, 'a.b.*')
   end
 
   it "SQL::Blob should be aliasable and castable by default" do
@@ -1144,7 +1115,7 @@ describe "Sequel::SQLTime" do
     @db = Sequel.mock
   end
   after do
-    Sequel::SQLTime.date = nil
+    Sequel::application_timezone = Sequel::SQLTime.date = nil
   end
 
   it ".create should create from hour, minutes, seconds and optional microseconds" do
@@ -1152,6 +1123,14 @@ describe "Sequel::SQLTime" do
     @db.literal(Sequel::SQLTime.create(1, 2, 3, 500000)).must_equal "'01:02:03.500000'"
   end
 
+  it ".create should use utc is that is the application_timezone setting" do
+    Sequel::SQLTime.create(1, 2, 3).utc?.must_equal false
+    Sequel::application_timezone = :local
+    Sequel::SQLTime.create(1, 2, 3).utc?.must_equal false
+    Sequel::application_timezone = :utc
+    Sequel::SQLTime.create(1, 2, 3).utc?.must_equal true
+  end
+
   it ".create should use today's date by default" do
     Sequel::SQLTime.create(1, 2, 3).strftime('%Y-%m-%d').must_equal Date.today.strftime('%Y-%m-%d')
   end
